package cn.com.infosec.crypto.encodings;

import cn.com.infosec.crypto.AsymmetricBlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.Digest;
import cn.com.infosec.crypto.InvalidCipherTextException;
import cn.com.infosec.crypto.digests.SHA1Digest;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import java.security.SecureRandom;

public class OAEPEncoding implements AsymmetricBlockCipher {
   private byte[] defHash;
   private Digest hash;
   private AsymmetricBlockCipher engine;
   private SecureRandom random;
   private boolean forEncryption;

   public OAEPEncoding(AsymmetricBlockCipher cipher) {
      OAEPEncoding var10000 = this;
      SHA1Digest var2;
      var2 = new SHA1Digest.<init>();
      var10000.<init>(cipher, var2, (byte[])null);
   }

   public OAEPEncoding(AsymmetricBlockCipher cipher, Digest hash) {
      this(cipher, hash, (byte[])null);
   }

   public OAEPEncoding(AsymmetricBlockCipher cipher, Digest hash, byte[] encodingParams) {
      this.engine = cipher;
      this.hash = hash;
      this.defHash = new byte[hash.getDigestSize()];
      if (encodingParams != null) {
         int cipher1 = encodingParams.length;
         hash.update(encodingParams, 0, cipher1);
      }

      hash.doFinal(this.defHash, 0);
   }

   private void ItoOSP(int i, byte[] sp) {
      sp[0] = (byte)(i >>> 24);
      sp[1] = (byte)(i >>> 16);
      sp[2] = (byte)(i >>> 8);
      sp[3] = (byte)(i >>> 0);
   }

   private byte[] maskGeneratorFunction1(byte[] Z, int zOff, int zLen, int length) {
      byte[] var5 = new byte[length];
      byte[] var6 = new byte[this.defHash.length];
      byte var7 = 4;
      byte[] var8 = new byte[4];
      int var9 = 0;
      this.hash.reset();

      byte[] var15;
      do {
         this.ItoOSP(var9, var8);
         this.hash.update(Z, zOff, zLen);
         this.hash.update(var8, 0, var7);
         this.hash.doFinal(var6, 0);
         byte[] var10001 = this.defHash;
         int var10 = var9 * var10001.length;
         int var11 = var10001.length;
         System.arraycopy(var6, 0, var5, var10, var11);
      } while(++var9 < length / (var15 = this.defHash).length);

      if (var9 * var15.length < length) {
         this.ItoOSP(var9, var8);
         this.hash.update(Z, zOff, zLen);
         this.hash.update(var8, 0, var7);
         this.hash.doFinal(var6, 0);
         byte[] var12;
         int Z1 = var9 * (var12 = this.defHash).length;
         int var13 = length - var9 * var12.length;
         System.arraycopy(var6, 0, var5, Z1, var13);
      }

      return var5;
   }

   public AsymmetricBlockCipher getUnderlyingCipher() {
      return this.engine;
   }

   public void init(boolean forEncryption, CipherParameters param) {
      if (param instanceof ParametersWithRandom) {
         this.random = ((ParametersWithRandom)param).getRandom();
      } else {
         SecureRandom var3;
         var3 = new SecureRandom.<init>();
         this.random = var3;
      }

      this.engine.init(forEncryption, param);
      this.forEncryption = forEncryption;
   }

   public int getInputBlockSize() {
      int var1 = this.engine.getInputBlockSize();
      return this.forEncryption ? var1 - 1 - this.defHash.length * 2 : var1;
   }

   public int getOutputBlockSize() {
      int var1 = this.engine.getOutputBlockSize();
      return this.forEncryption ? var1 : var1 - 1 - this.defHash.length * 2;
   }

   public byte[] processBlock(byte[] in, int inOff, int inLen) {
      return this.forEncryption ? this.encodeBlock(in, inOff, inLen) : this.decodeBlock(in, inOff, inLen);
   }

   public byte[] encodeBlock(byte[] in, int inOff, int inLen) {
      int var4;
      int var10002 = var4 = this.getInputBlockSize() + 1 + this.defHash.length * 2;
      byte[] var5;
      byte[] var10007 = var5 = new byte[var4];
      int var6;
      int var10008 = var6 = var4 - inLen;
      System.arraycopy(in, inOff, var5, var6, inLen);
      var10007[var10008 - 1] = 1;
      byte[] var10006 = this.defHash;
      int in1 = var10006.length;
      inOff = var10006.length;
      System.arraycopy(var10006, 0, var5, in1, inOff);
      byte[] inOff1 = new byte[in1 = this.defHash.length];
      this.random.nextBytes(inOff1);
      inLen = var10002 - this.defHash.length;
      in = this.maskGeneratorFunction1(inOff1, 0, in1, inLen);

      for(inLen = this.defHash.length; inLen != var4; ++inLen) {
         var5[inLen] ^= in[inLen - this.defHash.length];
      }

      in1 = this.defHash.length;
      System.arraycopy(inOff1, 0, var5, 0, in1);
      byte[] var10 = this.defHash;
      in1 = var10.length;
      inOff = var4 - var10.length;
      inLen = var10.length;
      byte[] var9 = this.maskGeneratorFunction1(var5, in1, inOff, inLen);

      for(in1 = 0; in1 != this.defHash.length; ++in1) {
         var5[in1] ^= var9[in1];
      }

      return this.engine.processBlock(var5, 0, var4);
   }

   public byte[] decodeBlock(byte[] in, int inOff, int inLen) {
      byte[] var10000;
      if ((in = this.engine.processBlock(in, inOff, inLen)).length < this.engine.getOutputBlockSize()) {
         var10000 = in;
         byte[] var10001 = in;
         int var10002 = this.engine.getOutputBlockSize();
         byte[] var10003 = in;
         in = new byte[var10002];
         inOff = var10002 - var10003.length;
         inLen = var10001.length;
         System.arraycopy(var10000, 0, in, inOff, inLen);
      }

      byte[] inOff1;
      if (in.length < (inOff1 = this.defHash).length * 2 + 1) {
         throw new InvalidCipherTextException("data too short");
      } else {
         inLen = inOff1.length;
         inOff = in.length - inOff1.length;
         int var4 = inOff1.length;
         inOff1 = this.maskGeneratorFunction1(in, inLen, inOff, var4);

         byte[] var7;
         for(inLen = 0; inLen != (var7 = this.defHash).length; ++inLen) {
            in[inLen] ^= inOff1[inLen];
         }

         inOff = var7.length;
         inLen = in.length - var7.length;
         var7 = this.maskGeneratorFunction1(in, 0, inOff, inLen);

         for(inOff = this.defHash.length; inOff != in.length; ++inOff) {
            in[inOff] ^= var7[inOff - this.defHash.length];
         }

         byte[] inLen1;
         for(inOff = 0; inOff != (inLen1 = this.defHash).length; ++inOff) {
            if (inLen1[inOff] != in[inLen1.length + inOff]) {
               throw new InvalidCipherTextException("data hash wrong");
            }
         }

         int var5;
         for(var5 = inLen1.length * 2; var5 != in.length && in[var5] != 1 && in[var5] == 0; ++var5) {
         }

         if (var5 < in.length - 1 && in[var5] == 1) {
            ++var5;
            var10000 = inLen1 = new byte[inOff = in.length - var5];
            System.arraycopy(in, var5, inLen1, 0, inOff);
            return var10000;
         } else {
            throw new InvalidCipherTextException("data start wrong " + var5);
         }
      }
   }
}
